;
;    Function C:PNEUMSIG
;
; Draws a pneumatic signal line for Instrument Loop Diagrams
;
; Designed and implemented by Larry V. Glover  May ,1987
; 
; Prompts for series of points for the pneumatic signal lines.
; Line symbols are placed starting from the center of the line
; and trims for end of line clear space.
;
(setq MOT "(c)1987 - PNEUMATIC SIGNAL LINE - LARRY V. GLOVER (vers 1.0)") ;motto
(defun PSTICK (pt ag ag2)                      ;Define center pts for tick marks
      (stick pt ag ag2 0.025 0.12)                 ;Pass distance for center pt
      (stick pt ag ag2 -0.025 0.12)                ; and end pts of t.marks
)
(defun STICK (pt ag ag2 idis1 idis2)          ;Place end pts and draw tick marks
      (setq spt1 (polar pt ag idis1)               ;Place center pt of tick mark
            spt2 (polar spt1 ag2 idis2)            ;Place one end of tick mark
            spt3 (polar spt1 ag2 (- 0 idis2))      ;Place other end of tick mark
      ) 
      (command "line" spt2 spt3 "")                   ;Draw tick mark
)
(defun PLPTS (ds)                                     ;Place incremental ct pts
      (setq rm1 (rem
                (/ (distance a b) 2) (abs ds)))       ;Get line end clear space
      (if (< rm1 (* 0.40 (abs ds)))           ;Test length line end clear space
          (setq np (1- (fix (/ (/ (distance a b) 2) 0.5))))   ;Backup one half
          (setq np (fix (/ (/ (distance a b) 2) 0.5)))        ;number of t.marks
      )
      (while (> np 0)                                 ;Place pts at spacing
             (setq pt2 (polar pt1 ang1 ds))           ;Place next pt at spacing
             (pstick pt2 ang1 ang2)                   ;Draw tick marks
             (setq pt1 pt2)                    ;Reset pt1 to new pt for looping
             (setq np (1- np))                        ;Increment counter
      )
)
(defun C:PNEUMSIG ()                            ;Function to draw pneumatic line
   (setq noecho (getvar "cmdecho")              ;Save existing screen echo mode 
         bmode (getvar "blipmode")              ;Save existing blipmode
         a (getpoint "Pick point")              ;Get first point
         ptlist (list a)                        ;Start list of points
   )
   (setvar "cmdecho" 0)                         ;Turn screen echo off
   (while (/= a nil)                            ;Continue getting points and
          (setq a (getpoint "Pick next point")  ;   add them to the list
                ptlist (cons a ptlist)              
          )
   )
   (setq ptlist (reverse ptlist)                ;Reverse list 
         ptlen (- (length ptlist) 2)            ;Set counter
         count 0                                ;  for following loop
   )
   (while (/= count ptlen)                      ;Loop to pull list apart
          (setq a (nth count ptlist)            ;Set variables for main line
                b (nth (1+ count) ptlist)
          )
          (command "line" a b "")               ;Main component of pneumatic line
          (setq ang1 (angle a b)                ;Set angle of line
                pt1 (polar a ang1 (/ (distance a b) 2)) ;Strt center pt of line
                rpt pt1                         ;Save center pt
          )
          (cond                                 ;Test angle of line and set
               ((< ang1 (/ Pi 4))               ;     angles for tick marks
                (setq ang2 (+ ang1 (/ Pi 4)))
               )
               ((and (> ang1 (/ Pi 4)) (< ang1 (/ (* 3 Pi) 4)))
                (setq ang2 (- ang1 (/ Pi 4)))
               )
               ((and (> ang1 (/ (* Pi 3) 4)) (< ang1 (/ (* Pi 5) 4)))
                (setq ang2 (+ ang1 (/ Pi 4)))
               )
               ((and (> ang1 (/ (* Pi 5) 4)) (< ang1 (/ (* Pi 7) 4)))
                (setq ang2 (- ang1 (/ Pi 4)))
               )
               ((> ang1 (/ (* Pi 7) 4))
                (setq ang2 (+ ang1 (/ Pi 4)))
               )
          )
          (setvar "blipmode" 0)                 ;Turn off pt marks
          (if (>= (distance a b) 0.40)       ;Test line length for tick marks
              (progn    
               (pstick pt1 ang1 ang2)           ;Function to draw tick marks
               (plpts 0.5)                      ;Increment marks 
               (setq pt1 rpt)                   ;Return to center pt
               (plpts -0.5)                     ;Increment in other direction
              )
          )
          (setq count (1+ count))               ;Step counter
   )
   (setvar "blipmode" bmode)                    ;Restore original blipmode
   (setvar "cmdecho" noecho)                    ;Turn screen echo on
   (terpri)                                     ;Line space
   mot                                          ;Motto
)
