;.................................................................

;QMESH.LSP CONTAINS ALL THE LISP ROUTINES FOR QUAD MESH GENERATION

;Author: T.N. Pandarinath

;..................................................................

; Generates quad mesh for fem - loops over all regions

(defun quadmesh ()
    (setq fr (open "region.dat" "w"))
    (setq fn (open "node.dat" "w"))
    (setvar "CMDECHO" 0)
    (setq flag 1)
    (setq nreg (getint "Enter No. of Regions: "))
    (prin1 nreg fr)
    (princ "\n" fr)
    (repeat nreg
           (qmesh)
           (princ "\n")
    )
    (close fr)
    (close fn)
)

;..................................................
; Generates quad mesh for fem

(defun qmesh ()
    (rstlayer)
    (getregion)
    (setlayer)
    (prompt "\nWAIT -> Computing ...")
    (edge12 e1 e2 nseg)
    (docurl e3 e4 nseg)
    (donodel pointl1 pointl2 curl mseg nseg)
    (prompt "\nWAIT -> Drawing ...")
    (drawmesh nodel mseg nseg)
)

;....................................................

; Resets layer to 0

(defun rstlayer ()
     (command "LAYER" "s" "0" "off" "mesh" "" )
)
;...........................................................
; Gets region

(defun getregion (/ i rn be regl)
    (command "OSNAP" "nea")
    (setq e1 (entsel "Show Edge #1: \n"))
    (setq e2 (entsel "Show Edge #2: \n"))
    (setq e3 (entsel "Show Edge #3: \n"))
    (setq e4 (entsel "Show Edge #4: \n"))
    (command "OSNAP" "")
    (setq mseg (getint "Enter Number of Rows(M-seg): "))
    (setq nseg (getint "Enter Number of Columns(N-seg): "))
    (setq regl (list mseg nseg))
    (setq i 0)
    (repeat 4
       (prin1 (setq i (1+ i)))
       (setq rn (getint "Enter Regn.#: "))
       (setq be (getint "Enter Bedg.#: "))
       (setq regl (append regl (list rn be)))
    )
    (prin1 regl fr)
    (princ "\n" fr)
)

;.............................................................

; Sets layer to 'mesh'

(defun setlayer ()
     (command "LAYER" "s" "mesh" "off" "0" "")
)
;...........................................................

; Builds point lists for edges 1 & 2
; returns pointl1 & pointl2

(defun edge12 (e1 e2 nseg)
    (edgedivide e1 nseg)
    (setq pointl1 pointl)
    (edgedivide e2 nseg)
    (setq pointl2 pointl)
)

;.....................................................

; Edge divide for e1 & e2
; returns pointl returned by ln/arc-divide function

(defun edgedivide (ename nseg / entdat enttype pt pt1 pt2 cpt rad sa
                   ea dir)
  (setq entdat (entget (car ename)))
  (setq enttype (cdr (assoc '0 entdat)))
  (if (= enttype "LINE")
    (progn
      (setq pt (cadr ename))
      (setq pt1 (cdr (assoc '10 entdat)))
      (setq pt2 (cdr (assoc '11 entdat)))
      (if (> (distance pt1 pt) (distance pt2 pt))
          (progn
  		(setq pt pt1)
  		(setq pt1 pt2)
		(setq pt2 pt)
  	     )
      )
      (lndivide pt1 pt2 nseg)
    ))
  (if (= enttype  "ARC")
    (progn
      (setq pt (cadr ename))
      (setq cpt (cdr (assoc '10 entdat)))
      (setq rad (cdr (assoc '40 entdat)))
      (setq sa  (cdr (assoc '50 entdat)))
      (setq ea  (cdr (assoc '51 entdat)))
      (setq pt1 (polar cpt sa rad))
      (setq pt2 (polar cpt ea rad))
      (if (> (distance pt1 pt) (distance pt2 pt))
          (setq dir -1)
          (setq dir +1))
      (arcdivide cpt rad sa ea nseg dir)
    ))
)
;..................................................

; Form curvature list for the two side edges
; returns curl (curvature list)

(defun docurl (e3 e4 nseg / curv3 curv4 enttype curi curv)
    (setq radv3 0.0)
    (setq curv3 0.0)
    (setq enttype (cdr (assoc '0 (entget (car e3)))))
    (if (= enttype "ARC")
        (progn 
           (curvature e3)
           (setq curv3 curv)
           (setq radv3 radv)
        ))
    (setq curv4 0.0)
    (setq radv4 0.0)
    (setq enttype (cdr (assoc '0 (entget (car e4)))))
    (if (= enttype "ARC")
        (progn
           (curvature e4)
           (setq curv4 curv)
           (setq radv4 radv)
        ))
    (setq curl (list curv3))
    (setq radl (list radv3))
    (setq curi (/ (- curv4 curv3) nseg))
    (setq radi (/ (- radv4 radv3) nseg))
    (repeat (- nseg 1)
        (setq curl (append curl (list (setq curv3 (+ curv3 curi)))))
        (setq radl (append radl (list (setq radv3 (+ radv3 radi)))))
    )
    (setq curl (append curl (list curv4)))
    (setq radl (append radl (list radv4)))
)
;..................................................

; Determines curvature (sign and  value) of an arc
; returns curv (curvature value)

(defun curvature (ename / entdat pt cpt rad sa ea pt1 pt2 dir)
    (setq entdat (entget (car ename)))
    (setq pt (cadr ename))
    (setq cpt (cdr (assoc '10 entdat)))
    (setq rad (cdr (assoc '40 entdat)))
    (setq sa  (cdr (assoc '50 entdat)))
    (setq ea  (cdr (assoc '51 entdat)))
    (setq pt1 (polar cpt sa rad))
    (setq pt2 (polar cpt ea rad))
    (if (> (distance pt pt1) (distance pt pt2))
        (setq dir -1)
        (setq dir +1))
    (setq curv (* (/ 1 rad) dir))
    (setq radv (* rad dir))
)
;...........................................................
; Form node list for a region

(defun donodel (pointl1 pointl2 curl mseg nseg / i pt1 pt2 cur x y
                         diff midpt d h rad cpt sa ea dir pointl)
    (setq nodel '())
    (setq i 0)
    (repeat (1+ nseg)
       (setq pt1 (nth i pointl1))
       (setq pt2 (nth i pointl2))
       (setq cur (nth i curl))
       (if (= cur 0)
           (lndivide pt1 pt2 mseg)
           (progn
              (setq x (/ (+ (car pt1) (car pt2)) 2))
              (setq y (/ (+ (cadr pt1) (cadr pt2)) 2))
              (setq midpt (list x y))
              (setq rad  (/ 1 (abs cur)))
              (setq d (/ (distance pt1 pt2) 2))
              (setq r1 rad)
              (setq d1 d)
              (if (> d rad) (setq rad (nth i radl)))
              (setq diff  (- (* rad rad) (* d d)))
              (if (< diff 1.0E-3) (setq diff 0.0))
              (setq h (sqrt diff))
              (if (> cur 0)
                (progn
                   (setq cpt (polar midpt (- (angle midpt pt1) (/ pi
                                                            2)) h))
                   (setq sa (angle cpt pt1))
                   (setq ea (angle cpt pt2))
                   (setq dir +1)
                )
                (progn
                   (setq cpt (polar midpt (+ (angle midpt pt1) (/ pi
                                                            2)) h))
                   (setq sa (angle cpt pt2))
                   (setq ea (angle cpt pt1))
                   (setq dir -1)
                )
              )
              (arcdivide cpt rad sa ea mseg dir)
           )
       )
       (setq i (+ 1 i))
       (setq nodel (append nodel (list pointl)))
       (prin1 pointl fn)
       (princ "\n"   fn)
    ) ; repeat ends
) ; donodel ends

;.........................................................
; Draws mesh

(defun drawmesh (nodel mseg nseg / i j ptl1 ptl2 pt1 pt2)
   (drawlnseg (nth 0 nodel) mseg)
   (setq i 0)
   (repeat nseg
      (setq ptl1 (nth i nodel))
      (setq ptl2 (nth (setq i (1+ i)) nodel))
      (drawlnseg ptl2 mseg)
      (setq j 0)
      (repeat (1+ mseg)
         (setq pt1 (nth j ptl1))
         (setq pt2 (nth j ptl2))
         (command "LINE" pt1 pt2)
         (command "")
         (setq j (1+ j))
      )
   )
)

;..........................................................

; Draws a series of line segments to generate mesh

(defun drawlnseg (pointl nseg / i  pt1 pt2)
    (setq i 0)
    (command "POINT" (nth 0 pointl))
    (repeat nseg
       (setq pt1 (nth i pointl))
       (setq pt2 (nth (setq i (1+ i))  pointl))
       (command "LINE" pt1 pt2 "")
       (command "POINT" pt2)
    )
)

;....................................................
; Divides a line joining PT1 to PT2 into NSEG segments
; returns pointl (point list)

(defun lndivide (pt1 pt2 nseg / x1 y1 x2 y2 l m ex ey)
	(setq x1 (car pt1))
	(setq y1 (cadr pt1))
	(setq x2 (car pt2))
	(setq y2 (cadr pt2))
	(setq l 1)
	(setq m (- nseg 1))
	(setq pointl (list pt1))
	(repeat (- nseg 1)
        (setq ex (/ (+ (* l x2) (* m x1)) (+ l m)))
        (setq ey (/ (+ (* l y2) (* m y1)) (+ l m)))
	   (setq pointl (append pointl (list (list ex ey))))
        (setq l (+ l 1))
        (setq m (- m 1))
     )
     (setq pointl (append pointl (list pt2)))
)
; ..........................................................

; Divides an arc into NSEG segments
; returns pointl

(defun arcdivide (cpt rad sa ea nseg dir / ia i x y diff ang pt)
    (setq diff (- ea sa))
    (if (< diff 0) (setq diff (+ diff (* 2 pi))))
    (setq ia (/ diff nseg))
    (setq i 0)
    (setq pointl '())
    (repeat (1+ nseg)
      (setq ang (+ sa (* ia i)))
      (if (> ang (* 2 pi)) (setq ang (- ang (* 2 pi))))
      (setq pt (polar cpt ang rad))
      (if (< (setq x (car pt)) 1.0E-6) (setq x 0.0))
      (if (< (setq y (cadr pt)) 1.0E-6) (setq y 0.0))
      (setq pointl (append pointl (list (list x y))))
      (setq i (1+ i))
    )
    (if (= dir -1)
        (setq pointl (reverse pointl)))
)
;...................................................

(defun Qmesh_Lsp_File_Loaded () nil )
;...................................................
