;SPELL.LSP - Extracts all text entities from drawing data
;base and puts in file named 'xxxxxxx.spl' then transfers to
;word processor for spelling check.  After exiting word processor
;program returns to Autocad and replaces lines of text in drawing
;that have been changed.
;
;The program depends on each line in the .spl file corresponding to
;a single text entity in the Autocad database.  The word processor
;must create a pure ASCII file with out the inserting any control 
;characters, extra line feeds, or extra spaces so make sure your line
;length in the word processor is set longer than your longest text line.
;Otherwise this can really make a mess of the drawing (which fortunately
;can be fixed by and Undo).  Any of these things will tend to substantially ;alter the length of the text line so the program will ask for verification
;before replacing lines where the length of the new line differs by
;more than one character. This will warn the operator when things get
;out of synch and a CTRL-C will stop the program.
;
;The program requires access to word processor by a command "WP" 
;defined in the ACAD.PGP which references an appropriate batch file.
;
;
(defun c:spell (/ file fname ename elist ol nl x)
   (setq fname (getstring "File name to use < .spl NR>: "))
   (setq file (open (strcat fname ".spl") "w"))
   (setq ename (entnext))
   (while (boundp 'ename)
      (setq elist (entget ename))
      (if (= (cdr (assoc 0 elist)) "TEXT")
         (write-line (cdr (assoc 1 elist)) file))
      (setq ename (entnext ename))
   )
   (close file)
   (setq ename ())
(command "WP"); Must be defined in ACAD.PGP
   (setq file (open (strcat fname ".spl") "r"))
   (setq ename (entnext))
   (while (boundp 'ename)
      (setq elist (entget ename))
      (if (= (cdr (assoc 0 elist)) "TEXT")
         (progn
            (setq ol (cdr (assoc 1 elist)) nl (read-line file) x 1)
            (if (/= ol nl)
               (progn
                  (if (< 1 (abs (- (strlen ol) (strlen nl))))
                     (progn
                        (print (strcat "Replace: " ol))
                        (print (strcat "   With: " nl))
                        (setq x (getint "\n       <1=Y or 2=N>? "))
                     )
                  )
                  (if (= x 1)
                     (progn
                        (setq elist (subst (cons 1 nl) (cons 1 ol) elist))
                        (entmod elist)
                     )
                  )
               )
            )
         )
      )
   (setq ename (entnext ename))
   )
(close file)
)
